/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: ErrorCode.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_ErrorCode_H
#define OpenSOAP_ErrorCode_H

/*
=begin
= OpenSOAP Error code definitions.
=end
 */

/* no error */
#define OPENSOAP_NO_ERROR (0)

/* not categorize error */
#define OPENSOAP_NOT_CATEGORIZE_ERROR   (0xffffffffL)

/* implementation error */
#define OPENSOAP_IMPLEMENTATION_ERROR   (0x40000000L)
#define OPENSOAP_YET_IMPLEMENTATION     (0x40000001L)

/* parameter bad value */
#define OPENSOAP_PARAMETER_BADVALUE     (0x20000001L)

/* user defined error */
#define OPENSOAP_USERDEFINE_ERROR       (0x10000000L)

/* memory error */
#define OPENSOAP_MEM_ERROR      (0x00010000L)
/* memory alloc error */
#define OPENSOAP_MEM_BADALLOC   (0x00010001L)
/* memory out of range */
#define OPENSOAP_MEM_OUTOFRANGE (0x00010002L)

/* char convert error */
#define OPENSOAP_CHAR_ERROR             (0x00020000L)
/* iconv not implement error */
#define OPENSOAP_ICONV_NOT_IMPL         (0x00020001L)
/* multi byte sequence invalid */
#define OPENSOAP_INVALID_MB_SEQUENCE    (0x00020002L)
/* multi byte sequence incomplete */
#define OPENSOAP_INCOMPLETE_MB_SEQUENCE (0x00020003L)
/* unknown charencoding */
#define OPENSOAP_UNKNOWN_CHARENCODE     (0x00020004L)
/* setlocale failure */
#define OPENSOAP_SETLOCALEFAILURE       (0x00020005L)

/* io error */
#define OPENSOAP_IO_ERROR       (0x00040000L)
/* read error */
#define OPENSOAP_IO_READ_ERROR	(0x00040002L)
/* write error */
#define OPENSOAP_IO_WRITE_ERROR	(0x00040002L)

/* file io error */
#define	OPENSOAP_FILE_ERROR		(0x00048000L)
/* file open error */
#define	OPENSOAP_FILEOPEN_ERROR	(0x00048001L)

/* xml manipulate error */
#define OPENSOAP_XML_ERROR       (0x00080000L)
/* #define OPENSOAP_XMLNODE_NOT_FOUND  (0x00080001L) */
#define OPENSOAP_XML_BADNAMESPACE (0x00080002L)
#define OPENSOAP_XML_NOHEADERBODY (0x00080003L)
#define OPENSOAP_XML_BADDOCUMENTTYPE (0x00080004L)
#define OPENSOAP_XML_BADMAKEDOCUMENT (0x00080005L)
#define OPENSOAP_XML_EMPTYDOCUMENT (0x00080006L)

/* security error */
#define OPENSOAP_SEC_ERROR       (0x00100000L)
/* key generate error */
#define OPENSOAP_SEC_KEYGEN_ERROR	(0x00100001L)
/* sign generate error */
#define OPENSOAP_SEC_SIGNGEN_ERROR	(0x00100002L)
/* sign verify error */
#define OPENSOAP_SEC_SIGNVERIFY_ERROR	(0x00100003L)
/* encrypt error */
#define OPENSOAP_SEC_ENCRYPT_ERROR	(0x00100004L)
/* decrypt error */
#define OPENSOAP_SEC_DECRYPT_ERROR	(0x00100005L)

/*
=begin
= OpenSOAP Error code judge macro.
=end
 */
#define OPENSOAP_SUCCEEDED(err) (!(err))
#define OPENSOAP_FAILED(err)  (err)

#endif  /* OpenSOAP_ErrorCode_H */
