/*-----------------------------------------------------------------------------
 * $RCSfile: ClientSocket.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: ClientSocket.c,v 1.21 2001/12/18 02:00:38 bandou Exp $";
#endif  /* _DEBUG */

#include "ClientSocket.h"

#include "AddrInfo.h"

/*
=begin
= OpenSOAP ClientSocket class
=end
 */

static
int
OpenSOAPClientSocketFree(OpenSOAPObjectPtr obj) {
    OpenSOAPSocketPtr s = (OpenSOAPSocketPtr)obj;
    int ret = OpenSOAPSocketReleaseMembers(s);

    if (OPENSOAP_SUCCEEDED(ret)) {
        free(obj);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPClientSocketCreate(s)
    OpenSOAP ClientSocket instance create

    :Parameters
      :OpenSOAPClientSocketPtr * [out] ((|s|))
        OpenSOAP ClientSocket pointer
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketCreate(OpenSOAPClientSocketPtr * /* [out] */ s) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    if (s) {
        *s = malloc(sizeof(OpenSOAPClientSocket));
        if (*s) {
            ret = OpenSOAPSocketInitialize((OpenSOAPSocketPtr)*s,
                                           OpenSOAPClientSocketFree,
                                           NULL, NULL);
            if (OPENSOAP_FAILED(ret)) {
                free(*s);
                *s = NULL;
            }
        }
        else {
            ret = OPENSOAP_MEM_BADALLOC;
        }
    }

    return ret;
}

      
/*
=begin
--- function#OpenSOAPClientSocketRelease(s)
    OpenSOAP ClientSocket JB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketRelease(OpenSOAPClientSocketPtr /* [in] */ s) {
    return OpenSOAPObjectRelease((OpenSOAPObjectPtr)s);
}

/*
=begin
--- function#OpenSOAPClientSocketOpen(s, node, service)
    OpenSOAP ClientSocket I[vBɐڑsB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
      :const char * [in] ((|node|))
        zXgB
        node. i.e. "www.isrc.co.jp", "192.168.0.1", ...
      :const char * [in] ((|service|))
        service. i.e. "http", "ftp", "smtp", "8080", ...
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketOpen(OpenSOAPClientSocketPtr /* [in] */ cs,
                         const char * /* [in] */ node,
                         const char * /* [in] */ service) {
    OpenSOAPSocketPtr s = (OpenSOAPSocketPtr)cs;
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (s && node && *node && service && *service) {
        OpenSOAPAddrInfoPtr res = NULL;
        ret = OpenSOAPAddrInfoCreate(node,
                                     service,
                                     &res);
        if (OPENSOAP_SUCCEEDED(ret)) {
            OpenSOAPAddrInfoPtr ai = res;
            for (; ai; ai = ai->ai_next) {
                ret = OpenSOAPSocketOpen(s,
                                         ai->ai_family,
                                         ai->ai_socktype,
                                         ai->ai_protocol);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    ret = OpenSOAPSocketConnect(s,
                                                ai->ai_addr,
                                                ai->ai_addrlen);
                    if (OPENSOAP_SUCCEEDED(ret)) {
                        break;
                    }
                    OpenSOAPSocketClose(s);
                }
            }
            
            OpenSOAPAddrInfoRelease(res);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPClientSocketClose(s)
    OpenSOAP ClientSocket N[YBڑؒfB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketClose(OpenSOAPClientSocketPtr /* [in] */ s) {
    int ret = OpenSOAPSocketClose((OpenSOAPSocketPtr)s);
    
    return ret;
}

/*
=begin
--- function#OpenSOAPClientSocketRead(s, buf, buf_sz)
    OpenSOAP ClientSocket ǂݍށB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
      :unsigned char * [out] buf,
        ǂݍ݃obt@̐擪|C^B
      :size_t * [in, out] buf_sz
        ǂݍ݃obt@̃TCYBďo͓ǂݍ񂾃TCYԂB
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketRead(OpenSOAPClientSocketPtr /* [in] */ s,
                   unsigned char * /* [out] */ buf,
                   size_t * /* [in, out] */ buf_sz) {
    int ret = OpenSOAPSocketRead((OpenSOAPSocketPtr)s,
                                 buf,
                                 buf_sz);

    return ret;
}

/*
=begin
--- function#OpenSOAPClientSocketWrite(s, buf, buf_sz)
    OpenSOAP ClientSocket ֏oB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
      :const unsigned char * [in] buf,
        ݃f[^̐擪|C^B
      :size_t * [in, out] buf_sz
        ݃f[^̃TCYBďo͏񂾃TCYԂB
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketWrite(OpenSOAPClientSocketPtr /* [in] */ s,
                    const unsigned char * /* [in] */ buf,
                    size_t * /* [in, out] */ buf_sz) {
    int ret = OpenSOAPSocketWrite((OpenSOAPSocketPtr)s,
                                  buf,
                                  buf_sz);

    return ret;
}

/*
=begin
--- function#OpenSOAPClientSocketReadToDelm(s, delm, delm_sz, buf)
    OpenSOAP ClientSocket f~^܂œǂݍށB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
      :const unsigned char * [in] delm
        f~^f[^B
      :size_t [in] delm_sz
        f~^f[^񒷂B
      :OpenSOAPByteArrayPtr [out] buf
        ǂݍ݃obt@B
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPClientSocketReadToDelm(OpenSOAPClientSocketPtr /* [in] */ s,
                         const unsigned char * /* [in] */ delm,
                         size_t /* [in] */ delm_sz,
                         OpenSOAPByteArrayPtr /* [out] */ buf) {
    int ret = OpenSOAPSocketReadToDelm((OpenSOAPSocketPtr)s,
                                       delm,
                                       delm_sz,
                                       buf);

    return ret;
}

/*
=begin
--- function#OpenSOAPClientSocketReadWithSize(s, sz, buf)
    OpenSOAP ClientSocket wTCYǂݍށBsz  0 ̏ꍇA
    ǂݍ߂邾ǂݍށB

    :Parameters
      :OpenSOAPClientSocketPtr  [in] ((|s|))
        OpenSOAP ClientSocket
      :size_t [in] ((|sz|))
        ǂݍ݃TCYB
      :OpenSOAPByteArrayPtr [out] ((|buf|))
        ǂݍ݃obt@B
    :Returns
      :int
      
=end
 */
#if 0
extern
int
/* OPENSOAP_API */
OpenSOAPClientSocketReadWithSize(OpenSOAPClientSocketPtr /* [in] */ s,
                                 size_t /* [in] */ sz,
                                 OpenSOAPByteArrayPtr /* [out] */ buf) {
    int ret = OpenSOAPStreamReadWithSize((OpenSOAPStreamPtr)s, sz, buf);

    return ret;
}
#endif

