/*-----------------------------------------------------------------------------
 * $RCSfile: Envelope.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: Envelope.c,v 1.68 2001/12/18 14:00:39 bandou Exp $";
#endif  /* _DEBUG */

#include "Envelope.h"
#include "Block.h"

#include <string.h>
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#include <libxml2/xmlmemory.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlmemory.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */



/* SOAP Envelope Element Name */
static
const char SOAPEnvelopeElementName[] = "Envelope";

/*
=begin
= OpenSOAP Envelope class
=end
 */

static
int
OpenSOAPEnvelopeReleaseMembers(OpenSOAPEnvelopePtr
                               /* [in, out] */ soap_env) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (soap_env) {
        OpenSOAPStringRelease(soap_env->version);

        ret = OpenSOAPXMLElmReleaseMembers((OpenSOAPXMLElmPtr)soap_env);
    }

    return ret;
}

static
int
OpenSOAPEnvelopeFree(OpenSOAPObjectPtr /* [in, out] */ obj) {
    int ret = OpenSOAPEnvelopeReleaseMembers((OpenSOAPEnvelopePtr)obj);

    if (OPENSOAP_SUCCEEDED(ret)) {
        free(obj);
    }

    return ret;
}

static
int
OpenSOAPEnvelopeInitialize(OpenSOAPEnvelopePtr /* [in, out] */ soap_env) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (soap_env) {
        soap_env->version = NULL;
        soap_env->header  = NULL;
        soap_env->body    = NULL;

        ret = OpenSOAPXMLElmInitialize((OpenSOAPXMLElmPtr)soap_env,
                                       OpenSOAPEnvelopeFree);
    }

    return ret;
}



/*
=begin
--- function#OpenSOAPEnvelopeCreate(soap_ver)
    OpenSOAP Envelope instance create
    
    :Parameters
      :OpenSOAPEnvelopePtr * [out] ((|soap_env|))
        Strage Buffer of OpenSOAP Envelope Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      SOAP Envelope ̈̊mۂƍ\̂̏s܂B
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeCreate(OpenSOAPEnvelopePtr *soap_env) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (soap_env) {
        ret = OPENSOAP_MEM_BADALLOC;
        *soap_env = malloc(sizeof(OpenSOAPEnvelope));
        if (*soap_env) {
            ret = OpenSOAPEnvelopeInitialize(*soap_env);

            if (OPENSOAP_FAILED(ret)) {
                free(*soap_env);
                *soap_env = NULL;
            }
        }
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeCreateString(soap_ver, env_prefix, soap_env)
    OpenSOAP Envelope instance create(OpenSOAPString)

    :Parameters
      :OpenSOAPStringPtr [in] ((||soap_ver|))
        SOAP Virsion
      :OpenSOAPStringPtr [in] ((|env_prefix|))
        SOAP Envelope namespace prefix.
      :OpenSOAPEnvelopePtr * [out] ((|soap_env|))
        Strage Buffer of OpenSOAP Envelope Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      (2001/08/28)
	  SOAP Envelope쐬̈̊mۂsASOAP Versionɂ
	  eX萔̑s܂B
      soap_ver      NULL ȊȌꍇ "1.1" or "1.2"
                    NULL ̏ꍇ "1.1" ƓB
      env_prefix    NULL ȊȌꍇ͓eɏ]B
                    NULL ̏ꍇsoap_verɂȉɏ]B
                    "1.1" SOAP-ENV
                    "1.2" env
      (2001/10/11)
	  This Function Is Static Function
=end
*/
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPEnvelopeCreateString(OpenSOAPStringPtr soap_ver,
                             OpenSOAPStringPtr env_prefix,
                             OpenSOAPEnvelopePtr *soap_env) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    int compare_ver;

    OpenSOAPStringPtr compare_soap_ver_str = NULL;
    OpenSOAPStringPtr namespace_uri_str = NULL;
    OpenSOAPStringPtr namespace_prefix_str = NULL;

    ret = OpenSOAPStringCreateWithMB("1.1", &compare_soap_ver_str);
    if ( OPENSOAP_FAILED(ret) ){
        return( ret );
    }
    ret = OpenSOAPStringCompare(compare_soap_ver_str, soap_ver, &compare_ver);
    if ( OPENSOAP_FAILED(ret) ){
        OpenSOAPStringRelease(compare_soap_ver_str);
        return( ret );
    }

    if( compare_ver != 0 ){
        OpenSOAPStringPtr compare_soap_ver_str = NULL;
        ret = OpenSOAPStringCreateWithMB("1.2", &compare_soap_ver_str);
        if ( OPENSOAP_FAILED(ret) ){
            OpenSOAPStringRelease(compare_soap_ver_str);
            return( ret );
        }
        ret = OpenSOAPStringCompare(compare_soap_ver_str, soap_ver, &compare_ver);
        if ( OPENSOAP_FAILED(ret) ){
            OpenSOAPStringRelease(compare_soap_ver_str);
            return( ret );
        }
        if( compare_ver != 0 ){
            OpenSOAPStringRelease(compare_soap_ver_str);
            ret = OPENSOAP_PARAMETER_BADVALUE;
            return ret;
        }
    }
    OpenSOAPStringRelease(compare_soap_ver_str);

    ret = OpenSOAPEnvelopeCreate( soap_env );
    if ( OPENSOAP_FAILED(ret) ){
        return( ret );
    }

    ret = OpenSOAPStringCreate( &((*soap_env)->envElm.super.name) );
    if( OPENSOAP_FAILED(ret) ){
        return ret;
    }

    ret = OpenSOAPStringCreateWithMB("Envelope", &(*soap_env)->envElm.super.name);
    if ( OPENSOAP_FAILED(ret) ){
        return( ret );
    }

    compare_soap_ver_str = NULL;
    ret = OpenSOAPStringCreateWithMB("1.1", &compare_soap_ver_str);
    if ( OPENSOAP_FAILED(ret) ){
        return( ret );
    }
    ret = OpenSOAPStringCompare(compare_soap_ver_str, soap_ver, &compare_ver);
    if ( OPENSOAP_FAILED(ret) ){
        OpenSOAPStringRelease(compare_soap_ver_str);
        return( ret );
    }
    OpenSOAPStringRelease(compare_soap_ver_str);
    if( compare_ver != 0 ){
        ret = OpenSOAPStringCreateWithMB( "http://schemas.xmlsoap.org/soap/envelope/", &namespace_uri_str );
        if( OPENSOAP_FAILED(ret) ) {
            return ret;
        }
        ret = OpenSOAPStringCreateWithMB( "SOAP-ENV", &namespace_prefix_str );
        if( OPENSOAP_FAILED(ret) ) {
            OpenSOAPStringRelease(namespace_uri_str);
            return ret;
        }
        ret = OpenSOAPXMLNamespaceCreateString(namespace_uri_str,
                                               namespace_prefix_str,
                                               &((*soap_env)->envElm.super.thisNamespace)); 
        OpenSOAPStringRelease(namespace_prefix_str);
        OpenSOAPStringRelease(namespace_uri_str);
        if(OPENSOAP_FAILED(ret)){
            return ret;
        }
    }
    else{
        ret = OpenSOAPStringCreateWithMB( "http://www.w3.org/2001/06/soap-envelope", &namespace_uri_str );
        if( OPENSOAP_FAILED(ret) ) {
            return ret;
        }
        ret = OpenSOAPStringCreateWithMB( "env", &namespace_prefix_str );
        if( OPENSOAP_FAILED(ret) ) {
            OpenSOAPStringRelease(namespace_uri_str);
            return ret;
        }
        ret = OpenSOAPXMLNamespaceCreateString( namespace_uri_str,
                                                namespace_prefix_str,
                                                &((*soap_env)->envElm.super.thisNamespace)); 
        OpenSOAPStringRelease(namespace_prefix_str);
        OpenSOAPStringRelease(namespace_uri_str);
        if(ret == OPENSOAP_PARAMETER_BADVALUE){
            return ret;
        }
    }
	
    ret = OpenSOAPStringCreate( &((*soap_env)->version) );
    if( OPENSOAP_FAILED(ret) ){
        return ret;
    }

    ret = OpenSOAPStringDuplicate((*soap_env)->version, &soap_ver);
    if( OPENSOAP_FAILED(ret)){
        return ret;
    }

    (*soap_env)->envElm.super.thisNamespace->definedElement = (OpenSOAPXMLElmPtr)soap_env;
    (*soap_env)->envElm.definedXMLNsList = (*soap_env)->envElm.super.thisNamespace;

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeCreateMB(soap_ver, env_prefix, soap_env)
    OpenSOAP Envelope instance create(MB)

    :Parameters
      :const cahr * [in] ((||soap_ver|))
        SOAP Virsion
      :const char * [in] ((|env_prefix|))
        SOAP Envelope namespace prefix.
      :OpenSOAPEnvelopePtr * [out] ((|soap_env|))
        Strage Buffer of OpenSOAP Envelope Pointer
    :Return value
      :int 
	    Return Code Is ErrorCode
    :Note
      (2001/08/28)
	  SOAP Envelope쐬̈̊mۂsASOAP Versionɂ
	  eX萔̑s܂B
      soap_ver      NULL ȊȌꍇ "1.1" or "1.2"
                    NULL ̏ꍇ "1.1" ƓB
      env_prefix    NULL ȊȌꍇ͓eɏ]B
                    NULL ̏ꍇsoap_verɂȉɏ]B
                    "1.1" SOAP-ENV
                    "1.2" env
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeCreateMB(const char *soap_ver,
                         const char *env_prefix,
                         OpenSOAPEnvelopePtr *soap_env) {
    static const char *SOAP_VERSIONS[] = {
        "1.1",
        "1.2",
        NULL
    };
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    const char **s_ver_i = SOAP_VERSIONS;

    if (!soap_ver) {
        soap_ver = SOAP_VERSIONS[0];
    }

    for (; *s_ver_i && strcmp(soap_ver, *s_ver_i) != 0; ++s_ver_i) {
    }
    if (!*s_ver_i) {
        return ret;
    }

    ret = OpenSOAPEnvelopeCreate( soap_env );
    if ( OPENSOAP_FAILED(ret) ){
        return ret;
    }

    ret = OpenSOAPStringCreate( &((*soap_env)->envElm.super.name) );
    if( OPENSOAP_FAILED(ret) ){
        return ret;
    }

    ret = OpenSOAPStringSetStringMB((*soap_env)->envElm.super.name, "Envelope");
    if(ret == OPENSOAP_PARAMETER_BADVALUE){
        return ret;
    }


    if( !(strcmp( soap_ver, "1.1" )) ){
        ret = OpenSOAPXMLNamespaceCreateMB( "http://schemas.xmlsoap.org/soap/envelope/",
                                            "SOAP-ENV",
                                            &((*soap_env)->envElm.super.thisNamespace)); 
        if(OPENSOAP_FAILED(ret)){
            return ret;
        }
    }
    else{
        ret = OpenSOAPXMLNamespaceCreateMB( "http://www.w3.org/2001/06/soap-envelope",
                                            "env",
                                            &((*soap_env)->envElm.super.thisNamespace)); 
        if(ret == OPENSOAP_PARAMETER_BADVALUE){
            return ret;
        }
    }
	
    ret = OpenSOAPStringCreate( &((*soap_env)->version) );
    if( OPENSOAP_FAILED(ret) ){
        return ret;
    }

    ret = OpenSOAPStringSetStringMB((*soap_env)->version, soap_ver);
    if( OPENSOAP_FAILED(ret)){
        return ret;
    }

    (*soap_env)->envElm.super.thisNamespace->definedElement = (OpenSOAPXMLElmPtr)soap_env;
    (*soap_env)->envElm.definedXMLNsList = (*soap_env)->envElm.super.thisNamespace;

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeCreateWC(soap_ver, env_prefix, soap_env)
    OpenSOAP Envelope instance create(WC)

    :Parameters
      :const cahr * [in] ((||soap_ver|))
        SOAP Virsion
      :const char * [in] ((|env_prefix|))
        SOAP Envelope namespace prefix.
      :OpenSOAPEnvelopePtr * [out] ((|soap_env|))
        Strage Buffer of OpenSOAP Envelope Pointer
    :Return value
      :type int
        Error code.
    :Note
      (2001/08/28)
	  SOAP Envelope쐬̈̊mۂsASOAP Versionɂ
	  eX萔̑s܂B
      soap_ver      NULL ȊȌꍇ "1.1" or "1.2"
                    NULL ̏ꍇ "1.1" ƓB
      env_prefix    NULL ȊȌꍇ͓eɏ]B
                    NULL ̏ꍇsoap_verɂȉɏ]B
                    "1.1" SOAP-ENV
                    "1.2" env
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeCreateWC(const wchar_t *soap_ver,
                         const wchar_t *env_prefix,
                         OpenSOAPEnvelopePtr *soap_env) {
	OpenSOAPStringPtr soap_ver_str = NULL;
	int ret = soap_ver
            ? OpenSOAPStringCreateWithWC(soap_ver, &soap_ver_str)
            : OPENSOAP_NO_ERROR;
	if (OPENSOAP_SUCCEEDED(ret)) {
		OpenSOAPStringPtr env_prefix_str = NULL;
		int ret = OpenSOAPStringCreateWithWC(env_prefix, &env_prefix_str);
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = OpenSOAPEnvelopeCreateString(soap_ver_str, env_prefix_str, soap_env);
			OpenSOAPStringRelease(env_prefix_str);
		}
		OpenSOAPStringRelease(soap_ver_str);
	}
	return ret;
}

/*
 */
static
const unsigned char *
searchXMLDocBody(/* [in] */ const unsigned char *xmldocBegin,
				 /* [in] */ size_t xmldocSize) {
	static const unsigned char XML_PROC_END[] = "?>";
	static const unsigned char *XML_PROC_END_END
		= XML_PROC_END + sizeof(XML_PROC_END) - 1;

	const unsigned char *ret = xmldocBegin;
	if (xmldocBegin && xmldocSize) {
		const unsigned char *xmldocEnd = xmldocBegin + xmldocSize;
		const unsigned char *xmlProcEndI = XML_PROC_END;
		for (; xmlProcEndI != XML_PROC_END_END
				 && ret != xmldocEnd; ++ret) {
			if (*ret == *xmlProcEndI) {
				++xmlProcEndI;
			}
			else {
				xmlProcEndI = XML_PROC_END;
			}
		}
		if (ret == xmldocEnd) {
			ret = xmldocBegin;
		}
	}

	return ret;
}

/*
 */
static
int
OpenSOAPByteArrayRemoveXMLProcAttr(/* [in]  */ OpenSOAPByteArrayPtr xmldoc,
								   /* [out] */ char **buf,
								   /* [out] */ size_t *bufSz) {
	static const char DEFAULT_XML_PROC[] = "<?xml version=\"1.0\"?>";
	static const size_t DEFAULT_XML_PROC_SIZE
		= sizeof(DEFAULT_XML_PROC) - 1;
	int ret = OPENSOAP_PARAMETER_BADVALUE;

	if (xmldoc && buf && bufSz) {
		const unsigned char *xmldocBegin = NULL;
		size_t xmldocSize = 0;
		*buf = NULL;
		*bufSz = 0;
		ret = OpenSOAPByteArrayGetBeginSizeConst(xmldoc,
												 &xmldocBegin,
												 &xmldocSize);
		if (OPENSOAP_SUCCEEDED(ret)) {
			const unsigned char *xmldocBody 
				= searchXMLDocBody(xmldocBegin, xmldocSize);
			size_t xmldocBodySize = xmldocSize - (xmldocBody - xmldocBegin);
			*bufSz = DEFAULT_XML_PROC_SIZE + xmldocBodySize;
			*buf = malloc(*bufSz);
			if (*buf) {
				memcpy(*buf, DEFAULT_XML_PROC, DEFAULT_XML_PROC_SIZE);
				memcpy(*buf + DEFAULT_XML_PROC_SIZE,
					   xmldocBody, xmldocBodySize);
			}
			else {
				*bufSz = 0;
				ret = OPENSOAP_MEM_BADALLOC;
			}
		}
	}
	
	return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeCreateCharEncoding(ch_enc, b_ary, soap_env)
    create char encoding
    
    :Parameters
      :const cahr * [in] ((|ch_enc|))
        character encoding
      :OpenSOAPByteArrayPtr [in] ((|b_ary|))
        OpenSOAP ByteArray
      :OpenSOAPEnvelopePtr * [out] ((|soap_env|))
        Strage Buffer of OpenSOAP Envelope Pointer
    :Return value
      :type int
        error code.
    :Note
      (2001/08/28)
      b_ary̓ech_enc(EUC-JP/SHIFT_JIS/UTF-8...)ɃGR[fBOB
      soap_envɊeXp[^b_aryݒ肷B
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeCreateCharEncoding(const char *ch_enc,
                                   OpenSOAPByteArrayPtr b_arry,
                                   OpenSOAPEnvelopePtr *soap_env) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	#define OPENSOAP_VER_LEN	5
	#define OPENSOAP_ENV_PRE	20

    xmlDocPtr	doc = NULL;
    xmlNsPtr	envNs = NULL;
    xmlNodePtr	envNode = NULL;
	xmlNodePtr	hbNode = NULL;
	xmlNodePtr	hbChildNode = NULL;
	xmlNodePtr	hbChildNextNode = NULL;
	xmlAttrPtr	nextAttr = NULL;
	xmlNsPtr	nextNs = NULL;
	int  isBlankNode;

	OpenSOAPStringPtr opensoap_nodevalue = NULL;
	OpenSOAPStringPtr attr_value = NULL;

	char soap_ver[OPENSOAP_VER_LEN];
	char env_prefix[OPENSOAP_ENV_PRE];

	OpenSOAPBlockPtr	 h_block = NULL;
	OpenSOAPBlockPtr	 b_block = NULL;

	OpenSOAPByteArrayPtr tmp_b_arry = NULL;
	OpenSOAPXMLAttrPtr new_attr = NULL;

	size_t flag_addHeaderAttr; /* 0:not added; 1:added */
	size_t flag_addHeaderNs;
	size_t flag_addBodyAttr;
	size_t flag_addBodyNs;


	/* change b_arry to "UTF-8" */
	ret = OpenSOAPByteArrayCreate(&tmp_b_arry);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPStringConvertCharEncoding(ch_enc, b_arry, "UTF-8", tmp_b_arry);
		if (OPENSOAP_SUCCEEDED(ret)) {
			char *tmpBuf = NULL;
			size_t tmpBufSize = 0;
			ret = OpenSOAPByteArrayRemoveXMLProcAttr(tmp_b_arry,
													 &tmpBuf,
													 &tmpBufSize);
			if (OPENSOAP_SUCCEEDED(ret)) {
				doc = xmlParseMemory(tmpBuf, tmpBufSize);
				free(tmpBuf);
			}
	   }
		ret = OpenSOAPByteArrayRelease( tmp_b_arry);
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
	}
    
    if (doc == NULL) {
		ret = OPENSOAP_XML_BADMAKEDOCUMENT;
		return ret;
	}
	
	/* check document (root node) */
    envNode = xmlDocGetRootElement(doc);
    if (envNode == NULL) {
		xmlFreeDoc(doc);
		ret = OPENSOAP_XML_EMPTYDOCUMENT;
		return ret;
    }
	
	/* check namespace */
    envNs = xmlSearchNsByHref(doc, envNode,
						   "http://schemas.xmlsoap.org/soap/envelope/");
    if (envNs == NULL) {
		envNs = xmlSearchNsByHref(doc, envNode,
							   "http://www.w3.org/2001/06/soap-envelope");
		if( envNs == NULL ){
			xmlFreeDoc(doc);
			ret = OPENSOAP_XML_BADNAMESPACE;
			return ret;
		}
		else{
			strcpy( soap_ver, "1.2" );
			strcpy( env_prefix, envNs->prefix );
		}
    }
	else{
		strcpy( soap_ver, "1.1" );
		strcpy( env_prefix, envNs->prefix );
	}
	
	/* check document type (root node) */
    if (strcmp(envNode->name, "Envelope") != 0) {
		xmlFreeDoc(doc);
		ret = OPENSOAP_XML_BADDOCUMENTTYPE;
		return ret;
    }

	ret = OpenSOAPEnvelopeCreateMB( soap_ver, env_prefix, soap_env );
	if( OPENSOAP_FAILED(ret) ) {
		return ret;
	}

	/* add envelope's attribute */
	ret = OpenSOAPXMLElmAddAttrInCharEncoding( envNode,
											   (OpenSOAPXMLElmPtr)(*soap_env));
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	/* add header block */
	hbNode = envNode->xmlChildrenNode;
	isBlankNode = xmlIsBlankNode( hbNode );
	if( isBlankNode == 1 ) {
		hbNode = hbNode->next;
	}
	if( hbNode->ns == envNs ) {
		if( strcmp( hbNode->name, "Header" ) == 0 ) {
			hbChildNode = hbNode->xmlChildrenNode;
			isBlankNode = xmlIsBlankNode( hbChildNode );
			if( isBlankNode == 1 ) {
				hbChildNode = hbChildNode->next;
			}
			hbChildNextNode = hbChildNode;

			/* add header element only without header block */
			if( hbChildNextNode == NULL ){
				ret = OpenSOAPEnvelopeAddHeaderBlockMB( *soap_env,
													    NULL,
													    &h_block );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
				/* add header element attribute */
				ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbNode, (*soap_env)->header );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
				/* add header element namespace */
				ret = OpenSOAPXMLElmAddNsInCharEncoding( hbNode, (*soap_env)->header );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
			}


			/* add header block */
			flag_addHeaderAttr = 0;
			flag_addHeaderNs = 0;
			while( hbChildNextNode != NULL ) {
				OpenSOAPByteArrayPtr mb_name = NULL;
				const unsigned char *mb_name_beg = NULL;

				ret = OpenSOAPByteArrayCreate( &mb_name );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				ret = OpenSOAPXMLElmConvertEncodingFromUTF8Libxml( hbChildNextNode->name,
																	 mb_name,
																	 &mb_name_beg );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				ret = OpenSOAPEnvelopeAddHeaderBlockMB( *soap_env,
														mb_name_beg,
														&h_block );
				if( OPENSOAP_FAILED(ret) ) {
					return ret;
				}

				ret = OpenSOAPByteArrayRelease( mb_name );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				/* add header element attribute */
				if( flag_addHeaderAttr == 0 ){
					ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbNode, (*soap_env)->header );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					flag_addHeaderAttr = 1;
				}

				/* add header element namespace */
				if( flag_addHeaderNs == 0 ){
					ret = OpenSOAPXMLElmAddNsInCharEncoding( hbNode, (*soap_env)->header );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					flag_addHeaderNs = 1;
				}

				/* add header block attributes */
				ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbChildNextNode, 
														   (OpenSOAPXMLElmPtr)h_block );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				/* add header block namespace */
				ret = OpenSOAPXMLElmAddNsInCharEncoding( hbChildNextNode,
														 (OpenSOAPXMLElmPtr)h_block );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				/* add header block's child node */
				if( hbChildNextNode->children != NULL ) {
					xmlNodePtr hbChildNextChild;
					hbChildNextChild = hbChildNextNode->children;
					isBlankNode = xmlIsBlankNode( hbChildNextChild );
					if( isBlankNode == 1 ){
						hbChildNextChild = hbChildNextChild->next;
					}
					if( strcmp( hbChildNextChild->name, "text" ) != 0 ){
						ret = OpenSOAPXMLElmCreateChildNodeFromLibxml( (OpenSOAPXMLElmPtr)h_block, hbChildNextNode );
						if( OPENSOAP_FAILED(ret) ) {
							return ret;
						}
					}
					else{
						OpenSOAPByteArrayPtr mb_content = NULL;
						const unsigned char *mb_content_beg = NULL;
						size_t mb_content_sz = 0;
						unsigned char *mb_content_beg_noLF = NULL; 
						size_t mb_content_sz_noLF = 0;
						int i;

						ret = OpenSOAPByteArrayCreate( &mb_content );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}
						ret = OpenSOAPXMLElmConvertEncodingFromUTF8Libxml( hbChildNextChild->content,
																			 mb_content,
																			 &mb_content_beg );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}

						/* delete '\t' '\n' '\r' in the content */
						ret = OpenSOAPByteArrayGetBeginSizeConst(mb_content, &mb_content_beg, &mb_content_sz);
						mb_content_beg_noLF = malloc( mb_content_sz * sizeof(unsigned char) );
						if( mb_content_beg_noLF == NULL ){
							ret = OPENSOAP_MEM_BADALLOC;
							return ret;
						}

						for( i=0; i<(int)mb_content_sz; i++ ){
							if( mb_content_beg[i] != '\n' && mb_content_beg[i] != '\r' && mb_content_beg[i] != '\t'){
								mb_content_beg_noLF[mb_content_sz_noLF] = mb_content_beg[i];
								mb_content_sz_noLF++;
							}
						}

						ret = OpenSOAPStringCreateWithMB( mb_content_beg_noLF, &opensoap_nodevalue );
						if( OPENSOAP_FAILED(ret) ) {
							free( mb_content_beg_noLF );
							return ret;
						}
						free( mb_content_beg_noLF );

						ret = OpenSOAPBlockSetValueMB( h_block, "string", &opensoap_nodevalue );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}
						ret = OpenSOAPByteArrayRelease( mb_content );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}
					}
				}

				hbChildNextNode = hbChildNextNode->next;
				isBlankNode = xmlIsBlankNode( hbChildNextNode );
				if( isBlankNode == 1 ) {
					hbChildNextNode = hbChildNextNode->next;
				}
			}

			/* add body block after header block*/
			hbNode = hbNode->next;
			if( hbNode == NULL ){
				ret = OPENSOAP_XML_NOHEADERBODY;
				return ret;
			}
			else{
				isBlankNode = xmlIsBlankNode( hbNode );
				if( isBlankNode == 1 ) {
					hbNode = hbNode->next;
				}
			}
			if( hbNode != NULL && strcmp( hbNode->name, "Body" ) == 0 ) {
				hbChildNode = hbNode->xmlChildrenNode;
				isBlankNode = xmlIsBlankNode( hbChildNode );
				if( isBlankNode == 1 ) {
					hbChildNode = hbChildNode->next;
				}
				hbChildNextNode = hbChildNode;

				/* add body element only without body block */
				if( hbChildNextNode == NULL ){
					ret = OpenSOAPEnvelopeAddBodyBlockMB( *soap_env,
														  NULL,
														  &b_block );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					/* add body element attribute */
					ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbNode, (*soap_env)->body );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					/* add body element namespace */
					ret = OpenSOAPXMLElmAddNsInCharEncoding( hbNode, (*soap_env)->body );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
				}

				/* add body block */
				flag_addBodyAttr = 0;
				flag_addBodyNs = 0;
				while( hbChildNextNode != NULL ) {
					OpenSOAPByteArrayPtr mb_name = NULL;
					const unsigned char *mb_name_beg = NULL;

					ret = OpenSOAPByteArrayCreate( &mb_name );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}

					ret = OpenSOAPXMLElmConvertEncodingFromUTF8Libxml( hbChildNextNode->name,
																		 mb_name,
																		 &mb_name_beg );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					ret = OpenSOAPEnvelopeAddBodyBlockMB( *soap_env,
														  mb_name_beg,
														  &b_block );
					if( OPENSOAP_FAILED(ret) ) {
						return ret;
					}

					/* add body element attribute */
					if( flag_addBodyAttr == 0 ){
						ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbNode, (*soap_env)->body );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}
						flag_addBodyAttr = 1;
					}

					/* add body element namespace */
					if( flag_addBodyNs == 0 ){
						ret = OpenSOAPXMLElmAddNsInCharEncoding( hbNode, (*soap_env)->body );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}
						flag_addBodyNs = 1;
					}

					/* add body block attributes */
					ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbChildNextNode,
															   (OpenSOAPXMLElmPtr)b_block);
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}

					/* add body block namespace */
					ret = OpenSOAPXMLElmAddNsInCharEncoding( hbChildNextNode,
															 (OpenSOAPXMLElmPtr)b_block);
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}

					/* add body block's child node */
					if( hbChildNextNode->children != NULL ) {
						xmlNodePtr hbChildNextChild;
						hbChildNextChild = hbChildNextNode->children;
						isBlankNode = xmlIsBlankNode( hbChildNextChild );
						if( isBlankNode == 1 ){
							hbChildNextChild = hbChildNextChild->next;
						}
						if( strcmp( hbChildNextChild->name, "text" ) != 0 ){
							ret = OpenSOAPXMLElmCreateChildNodeFromLibxml( (OpenSOAPXMLElmPtr)b_block, hbChildNextNode );
							if( OPENSOAP_FAILED(ret) ) {
								return ret;
							}
						}
						else{
							OpenSOAPByteArrayPtr mb_content = NULL;
							const unsigned char *mb_content_beg = NULL;
							size_t mb_content_sz = 0;
							unsigned char *mb_content_beg_noLF = NULL;
							size_t mb_content_sz_noLF = 0;
							int i;

							ret = OpenSOAPByteArrayCreate( &mb_content );
							if( OPENSOAP_FAILED(ret) ){
								return ret;
							}
							ret = OpenSOAPXMLElmConvertEncodingFromUTF8Libxml( hbChildNextChild->content,
																				 mb_content,
																				 &mb_content_beg );
							if( OPENSOAP_FAILED(ret) ){
								return ret;
							}

							/* delete '\t' '\n' '\r' in the content */
							ret = OpenSOAPByteArrayGetBeginSizeConst(mb_content, &mb_content_beg, &mb_content_sz);
							mb_content_beg_noLF = malloc( mb_content_sz * sizeof(unsigned char) );
							if( mb_content_beg_noLF == NULL ){
								ret = OPENSOAP_MEM_BADALLOC;
								return ret;
							}

							for( i=0; i<(int)mb_content_sz; i++ ){
								if( mb_content_beg[i] != '\n' && mb_content_beg[i] != '\r' && mb_content_beg[i] != '\t'){
									mb_content_beg_noLF[mb_content_sz_noLF] = mb_content_beg[i];
									mb_content_sz_noLF++;
								}
							}

							ret = OpenSOAPStringCreateWithMB( mb_content_beg_noLF, &opensoap_nodevalue );
							if( OPENSOAP_FAILED(ret) ) {
								free( mb_content_beg_noLF );
								return ret;
							}
							free(mb_content_beg_noLF);

							ret = OpenSOAPBlockSetValueMB( b_block, "string", &opensoap_nodevalue );
							if( OPENSOAP_FAILED(ret) ){
								return ret;
							}
							ret = OpenSOAPByteArrayRelease( mb_content );
							if( OPENSOAP_FAILED(ret) ){
								return ret;
							}
						}
					}

					hbChildNextNode = hbChildNextNode->next;
					isBlankNode = xmlIsBlankNode( hbChildNextNode );
					if( isBlankNode == 1 ) {
						hbChildNextNode = hbChildNextNode->next;
					}
				}
			}
			else {
				ret = OPENSOAP_XML_NOHEADERBODY;
				return ret;
			}

		}
		/* add body block without header block */
		else if( strcmp( hbNode->name, "Body" ) == 0 ) {
			hbChildNode = hbNode->xmlChildrenNode;
			isBlankNode = xmlIsBlankNode( hbChildNode );
			if( isBlankNode == 1 ) {
				hbChildNode = hbChildNode->next;
			}
			hbChildNextNode = hbChildNode;

			if( hbChildNextNode == NULL ){
				ret = OpenSOAPEnvelopeAddBodyBlockMB( *soap_env,
													  NULL,
													  &b_block );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
				/* add body element attribute */
				ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbNode, (*soap_env)->body );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
				/* add body element namespace */
				ret = OpenSOAPXMLElmAddNsInCharEncoding( hbNode, (*soap_env)->body );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
			}

			/* add body block */
			flag_addBodyAttr = 0;
			flag_addBodyNs = 0;
			while( hbChildNextNode != NULL ) {
				OpenSOAPByteArrayPtr mb_name = NULL;
				const unsigned char *mb_name_beg = NULL;

				ret = OpenSOAPByteArrayCreate( &mb_name );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				ret = OpenSOAPXMLElmConvertEncodingFromUTF8Libxml( hbChildNextNode->name,
																	 mb_name,
																	 &mb_name_beg );
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}
				ret = OpenSOAPEnvelopeAddBodyBlockMB( *soap_env,
													  mb_name_beg,
													  &b_block );
				if( OPENSOAP_FAILED(ret) ) {
					return ret;
				}

				/* add body element attribute */
				if( flag_addBodyAttr == 0 ){
					ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbNode, (*soap_env)->body );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					flag_addBodyAttr = 1;
				}
				/* add body element namespace */
				if( flag_addBodyNs == 0 ){
					ret = OpenSOAPXMLElmAddNsInCharEncoding( hbNode, (*soap_env)->body );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
					flag_addBodyNs = 1;
				}

				/* add body block attributes */
				ret = OpenSOAPXMLElmAddAttrInCharEncoding( hbChildNextNode,
														   (OpenSOAPXMLElmPtr)b_block);
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				/* add body block namespace */
				ret = OpenSOAPXMLElmAddNsInCharEncoding( hbChildNextNode,
														 (OpenSOAPXMLElmPtr)b_block);
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				/* add block's child node */
				if( hbChildNextNode->children != NULL ) {
					xmlNodePtr hbChildNextChild;
					hbChildNextChild = hbChildNextNode->children;
					isBlankNode = xmlIsBlankNode( hbChildNextChild );
					if( isBlankNode == 1 ){
						hbChildNextChild = hbChildNextChild->next;
					}
					if( strcmp( hbChildNextChild->name, "text" ) != 0 ){
						ret = OpenSOAPXMLElmCreateChildNodeFromLibxml( (OpenSOAPXMLElmPtr)b_block, hbChildNextNode );
						if( OPENSOAP_FAILED(ret) ) {
							return ret;
						}
					}
					else{
						OpenSOAPByteArrayPtr mb_content = NULL;
						const unsigned char *mb_content_beg = NULL;
						size_t mb_content_sz = 0;
						unsigned char *mb_content_beg_noLF = NULL;
						size_t mb_content_sz_noLF = 0;
						int i;

						ret = OpenSOAPByteArrayCreate( &mb_content );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}
						ret = OpenSOAPXMLElmConvertEncodingFromUTF8Libxml( hbChildNextChild->content,
																			 mb_content,
																			 &mb_content_beg );
						if( OPENSOAP_FAILED(ret) ){
							return ret;
						}

						/* delete '\t' '\n' '\r' in the content */
						ret = OpenSOAPByteArrayGetBeginSizeConst(mb_content,
																 &mb_content_beg,
																 &mb_content_sz);
						mb_content_beg_noLF = malloc(mb_content_sz * sizeof(unsigned char));
						if (mb_content_beg_noLF == NULL) {
							ret = OPENSOAP_MEM_BADALLOC;
							return ret;
						}

						for (i = 0; i < (int)mb_content_sz; ++i) {
							if (mb_content_beg[i] != '\n'
								&& mb_content_beg[i] != '\r'
								&& mb_content_beg[i] != '\t') {
								mb_content_beg_noLF[mb_content_sz_noLF]
									= mb_content_beg[i];
								++mb_content_sz_noLF;
							}
						}

						ret = OpenSOAPStringCreateWithMB(mb_content_beg_noLF,
														 &opensoap_nodevalue);
						if (OPENSOAP_FAILED(ret)) {
							free(mb_content_beg_noLF);
							return ret;
						}
						free(mb_content_beg_noLF);

						ret = OpenSOAPBlockSetValueMB(b_block,
													  "string",
													  &opensoap_nodevalue);
						if (OPENSOAP_FAILED(ret)) {
							return ret;
						}
						ret = OpenSOAPByteArrayRelease(mb_content);
						if (OPENSOAP_FAILED(ret)) {
							return ret;
						}
					}
				}

				hbChildNextNode = hbChildNextNode->next;
				isBlankNode = xmlIsBlankNode(hbChildNextNode);
				if (isBlankNode == 1) {
					hbChildNextNode = hbChildNextNode->next;
				}
			}
		}
		else {
			ret = OPENSOAP_XML_NOHEADERBODY;
			return ret;
		}
	}
	else {
		ret = OPENSOAP_XML_BADNAMESPACE;
		return ret;
	}

	ret = OPENSOAP_NO_ERROR;
	return ret;
}


/*
=begin
--- function#OpenSOAPEnvelopeRetain(soap_env)
    SOAP Envelope t@XǉB

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeRetain(OpenSOAPEnvelopePtr soap_env) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeRelease(soap_env)
    Release SOAP Envelope Buffer

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
	  SOAP Envelope쐬̈̊Js܂B
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeRelease(OpenSOAPEnvelopePtr soap_env) {
    int ret = OpenSOAPXMLElmRelease((OpenSOAPXMLElmPtr)soap_env);

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeAddHeaderBlockString(soap_env, block_name, h_block)
    Add SOAP Header Block(OpenSOAPString)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :OpenSOAPStringPtr [in] ((|block_name|))
        SOAP Block Name
      :OpenSOAPBlockPtr * [out] ((|h_block|))
        OpenSOAP Header Block
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      SOAP EnvelopëHeader Blockǉ܂B
      (2001/10/11)
      This Function Is Static Function
  
=end
*/
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPEnvelopeAddHeaderBlockString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                 OpenSOAPStringPtr /* [in] */ block_name,
                                 OpenSOAPBlockPtr * /* [out] */ h_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLElmPtr second_b;

	if( !soap_env || !h_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( !soap_env->header && block_name == NULL ){
		/* add body element only ( no child---body block )*/
		ret = OpenSOAPXMLElmCreate( &soap_env->header );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->header->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreateWithMB("Header", &soap_env->header->super.name);
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
		}
		else{
			second_b = (OpenSOAPXMLElmPtr)soap_env->envElm.super.children;
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
			soap_env->header->super.next = (OpenSOAPXMLNodePtr)second_b;
		}

		soap_env->header->super.parent = (OpenSOAPXMLNodePtr)soap_env;

		*h_block = NULL;

		ret = OPENSOAP_NO_ERROR;
		return ret;
	}

	if( !soap_env->header && block_name != NULL ){
		ret = OpenSOAPXMLElmCreate( &soap_env->header );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->header->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreateWithMB("Header", &soap_env->header->super.name);
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		
		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
		}
		else{
			second_b = (OpenSOAPXMLElmPtr)soap_env->envElm.super.children;
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
			soap_env->header->super.next = (OpenSOAPXMLNodePtr)second_b;
		}

		soap_env->header->super.parent = (OpenSOAPXMLNodePtr)soap_env;
	}

	ret = OpenSOAPBlockCreate( h_block );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	ret = OpenSOAPXMLElmAddChildString( soap_env->header, block_name, (OpenSOAPXMLElmPtr*)h_block );

    return ret;
}


/*
=begin
--- function#OpenSOAPEnvelopeAddHeaderBlockMB(soap_env, block_name, h_block)
    Add SOAP Header Block(MB)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const char * [in] ((|block_name|))
        SOAP Block Name
      :OpenSOAPBlockPtr * [out] ((|h_block|))
        OpenSOAP Header Block
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      SOAP EnvelopëHeader Blockǉ܂B
  
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeAddHeaderBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                 const char * /* [in] */ block_name,
                                 OpenSOAPBlockPtr * /* [out] */ h_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLElmPtr second_b;

	if( !soap_env || !h_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( !soap_env->header && block_name == NULL ){
		/* add body element only ( no child---body block )*/
		ret = OpenSOAPXMLElmCreate( &soap_env->header );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->header->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringSetStringMB(soap_env->header->super.name, "Header");
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
		}
		else{
			second_b = (OpenSOAPXMLElmPtr)soap_env->envElm.super.children;
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
			soap_env->header->super.next = (OpenSOAPXMLNodePtr)second_b;
		}

		soap_env->header->super.parent = (OpenSOAPXMLNodePtr)soap_env;
		soap_env->header->super.thisNamespace = soap_env->envElm.super.thisNamespace;

		*h_block = NULL;

		ret = OPENSOAP_NO_ERROR;
		return ret;
	}

	if( !soap_env->header && block_name != NULL ){
		ret = OpenSOAPXMLElmCreate( &soap_env->header );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->header->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringSetStringMB(soap_env->header->super.name, "Header");
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		
		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
		}
		else{
			second_b = (OpenSOAPXMLElmPtr)soap_env->envElm.super.children;
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->header;
			soap_env->header->super.next = (OpenSOAPXMLNodePtr)second_b;
		}

		soap_env->header->super.parent = (OpenSOAPXMLNodePtr)soap_env;
		soap_env->header->super.thisNamespace = soap_env->envElm.super.thisNamespace;
	}

	ret = OpenSOAPBlockCreate( h_block );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	ret = OpenSOAPXMLElmAddChildMB( soap_env->header, block_name, (OpenSOAPXMLElmPtr*)h_block );

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeAddHeaderBlockWC(soap_env, block_name, h_block)
    Add SOAP Header Block(WC)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const wchar_t * [in] ((|block_name|))
        SOAP Block Name
      :OpenSOAPBlockPtr * [out] ((|h_block|))
        OpenSOAP Header Block
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      SOAP EnvelopëHeader Blockǉ܂B
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeAddHeaderBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                 const wchar_t * /* [in] */ block_name,
                                 OpenSOAPBlockPtr * /* [out] */ h_block) {
	OpenSOAPStringPtr block_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(block_name, &block_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPEnvelopeAddHeaderBlockString(soap_env, block_name_str, h_block);
		OpenSOAPStringRelease(block_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetNextHeaderBlock(soap_env, h_block)
    Get SOAP Header Block

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :OpenSOAPBlockPtr * [in, out] ((|h_block|))
        if OpenSOAP Header Block. *h_block as in NULL then
        Return to First Header Block
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetNextHeaderBlock(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                   OpenSOAPBlockPtr * /* [in, out] */ h_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !h_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( soap_env->header != NULL ){
		ret = OpenSOAPXMLElmGetNextChild( soap_env->header,
										  (OpenSOAPXMLElmPtr *)(h_block) );
	}
	else{
		*h_block = NULL;
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetHeaderBlockString(soap_env, block_name, h_block)
    Get SOAP Header Block(OpenSOAPString)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :OpenSOAPStringPtr [in] ((|block_name|))
        SOAP Header Block name.
      :OpenSOAPBlockPtr * [out] ((|h_block|))
        Header Block return buffer's pointer.
    :Return value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
*/
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPEnvelopeGetHeaderBlockString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                               OpenSOAPStringPtr /* [in] */ block_name,
                               OpenSOAPBlockPtr * /* [out] */ h_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !h_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetChildString( soap_env->header, 
									block_name, 
									(OpenSOAPXMLElmPtr*)h_block );

    return ret;
}


/*
=begin
--- function#OpenSOAPEnvelopeGetHeaderBlockMB(soap_env, block_name, h_block)
    Get SOAP Header Block(MB)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const char * [in] ((|block_name|))
        SOAP Header Block name.
      :OpenSOAPBlockPtr * [out] ((|h_block|))
        Header Block return buffer's pointer.
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetHeaderBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                               const char * /* [in] */ block_name,
                               OpenSOAPBlockPtr * /* [out] */ h_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !h_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetChildMB( soap_env->header, 
									block_name, 
									(OpenSOAPXMLElmPtr*)h_block );

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetHeaderBlockWC(soap_env, block_name, h_block)
    Get SOAP Header Block(WC)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const wchar_t * [in] ((|block_name|))
        SOAP Header Block name.
      :OpenSOAPBlockPtr * [out] ((|h_block|))
        Header Block return buffer's pointer.
        
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetHeaderBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                               const wchar_t * /* [in] */ block_name,
                               OpenSOAPBlockPtr * /* [out] */ h_block) {
	OpenSOAPStringPtr block_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(block_name, &block_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPEnvelopeGetHeaderBlockString(soap_env, block_name_str, h_block);
		OpenSOAPStringRelease(block_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeAddBodyBlockString(soap_env, block_name, b_block)
    Add SOAP Body Block(OpenSOAPString)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :OpenSOAPStringPtr [in] ((|block_name|))
        SOAP Block Name
		If block_name==NULL, add body element only (no body block)
      :OpenSOAPBlockPtr * [out] ((|b_block|))
        OpenSOAP Body Block
        
    :Return value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
*/
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPEnvelopeAddBodyBlockString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                 OpenSOAPStringPtr /* [in] */ block_name,
                                 OpenSOAPBlockPtr * /* [out] */ b_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !b_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( !soap_env->body && block_name == NULL ){
		/* add body element only ( no child---body block )*/
		ret = OpenSOAPXMLElmCreate( &soap_env->body );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->body->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringSetStringMB(soap_env->body->super.name, "Body");
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->body;
		}
		else{
			soap_env->envElm.super.children->next = (OpenSOAPXMLNodePtr)soap_env->body;
			soap_env->body->super.prev = soap_env->envElm.super.children;
		}

		soap_env->body->super.parent = (OpenSOAPXMLNodePtr)soap_env;

		*b_block = NULL;
		
		ret = OPENSOAP_NO_ERROR;
		return ret;
	}
	
	if( !soap_env->body && block_name != NULL ){
		/* add body element and its child---body block named by block_name */
		ret = OpenSOAPXMLElmCreate( &soap_env->body );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->body->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringSetStringMB(soap_env->body->super.name, "Body");
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->body;
		}
		else{
			soap_env->envElm.super.children->next = (OpenSOAPXMLNodePtr)soap_env->body;
			soap_env->body->super.prev = soap_env->envElm.super.children;
		}
		soap_env->body->super.parent = (OpenSOAPXMLNodePtr)soap_env;
	}


	ret = OpenSOAPBlockCreate( b_block );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	ret = OpenSOAPXMLElmAddChildString( soap_env->body, block_name, (OpenSOAPXMLElmPtr*)b_block );

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeAddBodyBlockMB(soap_env, block_name, b_block)
    Add SOAP Body Block(MB)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const char * [in] ((|block_name|))
        SOAP Block Name
		If block_name==NULL, add body element only (no body block)
      :OpenSOAPBlockPtr * [out] ((|b_block|))
        OpenSOAP Body Block
        
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeAddBodyBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                 const char * /* [in] */ block_name,
                                 OpenSOAPBlockPtr * /* [out] */ b_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !b_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( !soap_env->body && block_name == NULL ){
		/* add body element only ( no child---body block )*/
		ret = OpenSOAPXMLElmCreate( &soap_env->body );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->body->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringSetStringMB(soap_env->body->super.name, "Body");
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->body;
		}
		else{
			soap_env->envElm.super.children->next = (OpenSOAPXMLNodePtr)soap_env->body;
			soap_env->body->super.prev = soap_env->envElm.super.children;
		}

		soap_env->body->super.parent = (OpenSOAPXMLNodePtr)soap_env;
		soap_env->body->super.thisNamespace = soap_env->envElm.super.thisNamespace;

		*b_block = NULL;
		
		ret = OPENSOAP_NO_ERROR;
		return ret;
	}
	
	if( !soap_env->body && block_name != NULL ){
		/* add body element and its child---body block named by block_name */
		ret = OpenSOAPXMLElmCreate( &soap_env->body );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringCreate( &(soap_env->body->super.name) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}

		ret = OpenSOAPStringSetStringMB(soap_env->body->super.name, "Body");
		if(ret == OPENSOAP_PARAMETER_BADVALUE){
			return ret;
		}

		if( soap_env->envElm.super.children == NULL ){
			soap_env->envElm.super.children = (OpenSOAPXMLNodePtr)soap_env->body;
		}
		else{
			soap_env->envElm.super.children->next = (OpenSOAPXMLNodePtr)soap_env->body;
			soap_env->body->super.prev = soap_env->envElm.super.children;
		}
		soap_env->body->super.parent = (OpenSOAPXMLNodePtr)soap_env;
		soap_env->body->super.thisNamespace = soap_env->envElm.super.thisNamespace;
	}


	ret = OpenSOAPBlockCreate( b_block );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	ret = OpenSOAPXMLElmAddChildMB( soap_env->body, block_name, (OpenSOAPXMLElmPtr*)b_block );

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeAddBodyBlockWC(soap_env, block_name, b_block)
    Add SOAP Body Block(WC)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const wchar_t * [in] ((|block_name|))
        SOAP Block Name
      :OpenSOAPBlockPtr * [out] ((|b_block|))
        OpenSOAP Body Block
        
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeAddBodyBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                 const wchar_t * /* [in] */ block_name,
                                 OpenSOAPBlockPtr * /* [out] */ b_block) {
	OpenSOAPStringPtr block_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(block_name, &block_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPEnvelopeAddBodyBlockString(soap_env, block_name_str, b_block);
		OpenSOAPStringRelease(block_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetNextBodyBlock(soap_env, b_block)
    Get SOAP Body Block

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :OpenSOAPBlockPtr * [in, out] ((|b_block|))
        if OpenSOAP Body Block. *b_block as in NULL then
        Return to First Body Block
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetNextBodyBlock(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                   OpenSOAPBlockPtr * /* [in, out] */ b_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !b_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( soap_env->body != NULL ){
		ret = OpenSOAPXMLElmGetNextChild( soap_env->body,
										  (OpenSOAPXMLElmPtr *)(b_block) );
	}
	else{
		*b_block = NULL;
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetBodyBlockString(soap_env, block_name, b_block)
    Get SOAP Body Block(OpenSOAPString)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :OpenSOAPStringPtr [in] ((|block_name|))
        SOAP Body Block name.
      :OpenSOAPBlockPtr * [out] ((|b_block|))
        Body Block return buffer's pointer.
        
    :Return value
      :int
	    Return Code Is ErrorCode

    :Note
      (2001/08/28)
      block_nameb_block߂B
	  (2001/10/11)
	  This Function Is Static Function
=end
*/
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPEnvelopeGetBodyBlockString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                               OpenSOAPStringPtr /* [in] */ block_name,
                               OpenSOAPBlockPtr * /* [out] */ b_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !b_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetChildString( soap_env->body, 
								  block_name, 
								 (OpenSOAPXMLElmPtr*)b_block );

    return ret;
}


/*
=begin
--- function#OpenSOAPEnvelopeGetBodyBlockMB(soap_env, block_name, b_block)
    Get SOAP Body Block(MB)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const char * [in] ((|block_name|))
        SOAP Body Block name.
      :OpenSOAPBlockPtr * [out] ((|b_block|))
        Body Block return buffer's pointer.
        
    :Return value
      :int
	    Return Code Is ErrorCode

    :Note
      (2001/08/28)
      block_nameb_block߂B
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetBodyBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                               const char * /* [in] */ block_name,
                               OpenSOAPBlockPtr * /* [out] */ b_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !b_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetChildMB( soap_env->body, 
									block_name, 
									(OpenSOAPXMLElmPtr*)b_block );

    return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetBodyBlockWC(soap_env, block_name, b_block)
    Get SOAP Body Block(WC)

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const wchar_t * [in] ((|block_name|))
        SOAP Body Block name.
      :OpenSOAPBlockPtr * [out] ((|b_block|))
        Body Block return buffer's pointer.
        
    :Return value
      :int
	    Return Code Is ErrorCode

    :Note
      (2001/08/28)
      block_nameb_block߂B
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetBodyBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                               const wchar_t * /* [in] */ block_name,
                               OpenSOAPBlockPtr * /* [out] */ b_block) {
	OpenSOAPStringPtr block_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(block_name, &block_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPEnvelopeGetBodyBlockString(soap_env, block_name_str, b_block);
		OpenSOAPStringRelease(block_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPEnvelopeGetCharEncodingString(soap_env, ch_enc, buf)
    SOAP Envelope character encoding output.

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP EnvelopeB
      :const char  * [in] ((|ch_enc|))
        character encoding. i.e. "EUC-JP", "SHIFT-JIS", "UTF-8"
      :OpenSOAPByteArrayPtr [out] ((|b_ary|))
        Strage Buffer
    :Return value
      :int
	    Return Code Is ErrorCode
 
    :Note
      (2001/08/28)
      soap_envSOAP MessagegݗĂB
     
=end
 */
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetCharEncodingString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                      const char * /* [in] */ ch_enc,
                                      OpenSOAPByteArrayPtr /* [out] */ b_ary) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	#define PROCLINE_SZ		80

	const char *xml_enc = (ch_enc) ? ch_enc : "UTF-8";

	char *procenv_char = NULL;
	size_t procenv_char_sz = 0;
	OpenSOAPByteArrayPtr procenv_barry = NULL;

	OpenSOAPByteArrayPtr tmp_b_ary = NULL;
	const unsigned char *tmp_b_ary_beg = NULL;
	size_t tmp_b_ary_sz = 0;


	/* get UTF-8 bytearray from soap_env */
	ret = OpenSOAPByteArrayCreate( &tmp_b_ary );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	ret = OpenSOAPXMLElmGetCharEncodingString( (OpenSOAPXMLElmPtr)soap_env, "UTF-8", tmp_b_ary );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	/* add processing line <?xml version="1.0", encoding="" ?>*/
	ret = OpenSOAPByteArrayGetBeginSizeConst( tmp_b_ary,
											  &tmp_b_ary_beg,
											  &tmp_b_ary_sz );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	procenv_char = malloc( (tmp_b_ary_sz+PROCLINE_SZ)*sizeof(char) );
	if( !procenv_char ){
		ret = OPENSOAP_MEM_BADALLOC;
		return ret;
	}

	strcpy( procenv_char, "<?xml version=\"1.0\" encoding=\"" );
	strcat( procenv_char, xml_enc );
	strcat( procenv_char, "\"?>\n" );
	strncat( procenv_char, tmp_b_ary_beg, tmp_b_ary_sz );
	procenv_char_sz = strlen( procenv_char );

	ret = OpenSOAPByteArrayCreate( &procenv_barry );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
    ret = OpenSOAPByteArraySetData(procenv_barry,
							       procenv_char,
                                   procenv_char_sz);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	/* change encoding from UTF-8 to ch_enc*/
 	ret = OpenSOAPStringConvertCharEncoding("UTF-8", procenv_barry, xml_enc, b_ary);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	if( procenv_char ){
		free( procenv_char );
	}

	if( tmp_b_ary != NULL ){
		ret = OpenSOAPByteArrayRelease(tmp_b_ary);
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
	}

	if( procenv_barry != NULL ){
		ret = OpenSOAPByteArrayRelease(procenv_barry);
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
	}

    return ret;
}



/*
=begin
--- function#OpenSOAPEnvelopeGetHeaderCharEncodingString(soap_env, ch_enc, b_ary)
    Soap Header character encoding output

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP Envelope.
      :const char  * [in] ((|ch_enc|))
        character encoding. i.e. "EUC-JP", "SHIFT-JIS", "UTF-8"
      :OpenSOAPByteArrayPtr [out] ((|b_ary|))
        Result Buffer        
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetHeaderCharEncodingString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
										    const char * /* [in] */ ch_enc,
										    OpenSOAPByteArrayPtr /* [out] */ b_ary) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !soap_env->header || !b_ary ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}
	else{
		ret = OpenSOAPXMLElmGetCharEncodingString( soap_env->header,
												   ch_enc,
												   b_ary );
	}

    return ret;
}



/*
=begin
--- function#OpenSOAPEnvelopeGetBodyCharEncodingString(soap_env, ch_enc, b_ary)
    Soap Body character encoding output

    :Parameters
      :OpenSOAPEnvelopePtr [in] ((|soap_env|))
        OpenSOAP Envelope.
      :const char  * [in] ((|ch_enc|))
        character encoding. i.e. "EUC-JP", "SHIFT-JIS", "UTF-8"
      :OpenSOAPByteArrayPtr [out] ((|b_ary|))
        Result Buffer        
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPEnvelopeGetBodyCharEncodingString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
										  const char * /* [in] */ ch_enc,
										  OpenSOAPByteArrayPtr /* [out] */ b_ary) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_env || !soap_env->body || !b_ary ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}
	else{
		ret = OpenSOAPXMLElmGetCharEncodingString( soap_env->body,
												   ch_enc,
												   b_ary );
	}

    return ret;
}

