/*-----------------------------------------------------------------------------
 * $RCSfile: XMLNode.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: XMLNode.c,v 1.20 2001/12/18 14:02:26 bandou Exp $";
#endif  /* _DEBUG */

#include "XMLNode.h"

/*
=begin
= OpenSOAP XMLNode base
=end
 */
int
OpenSOAPXMLNodeRelease(OpenSOAPXMLNodePtr /* [in, out] */ node) {
    int ret = OpenSOAPObjectRelease((OpenSOAPObjectPtr)node);

    return ret;
}

int
OpenSOAPXMLNodeReleaseList(OpenSOAPXMLNodePtr /* [in, out] */ node) {
    int	ret = OPENSOAP_NO_ERROR;

    while (node && OPENSOAP_SUCCEEDED(ret)) {
        OpenSOAPXMLNodePtr next = node->next;
            
        ret = OpenSOAPXMLNodeRelease(node);

        node = next;
    }

    return ret;
}

int
OpenSOAPXMLNodeReleaseMembers(OpenSOAPXMLNodePtr /* [in, out] */ node) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (node) {
        /* children release */
        OpenSOAPXMLNodeReleaseList(node->children);
        /* */
        OpenSOAPStringRelease(node->name);
        OpenSOAPStringRelease(node->value);

		ret = OPENSOAP_NO_ERROR;
    }
    
    return ret;
}

static
int
OpenSOAPXMLNodeFree(OpenSOAPObjectPtr /* [in, out] */ obj) {
    int ret = OpenSOAPXMLNodeReleaseMembers((OpenSOAPXMLNodePtr)obj);

    if (OPENSOAP_SUCCEEDED(ret)) {
        free(obj);
    }

    return ret;
}
    
/*
=begin
--- function#OpenSOAPXMLNodeInitialize(node)
    Initialize Of Node

    :Parameters
      :OpenSOAPXMLNodePtr [in, out] ((|node|))
        XML Node Pointer.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
int
OpenSOAPXMLNodeInitialize(OpenSOAPXMLNodePtr /* [in, out] */ node,
                          OpenSOAPObjectFreeFunc /* [in] */ free_func) {

    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (node) {
        node->prev = NULL;
        node->next = NULL;
        node->parent = NULL;
        node->children = NULL;

        node->name = NULL;
        node->value = NULL;
        node->thisNamespace = NULL;

        ret = OpenSOAPObjectInitialize((OpenSOAPObjectPtr)node,
                                       free_func ? free_func
                                       : OpenSOAPXMLNodeFree);
    }
    
    return ret;
}
      
/*
=begin
--- function#OpenSOAPXMLNodeSetName(node, name, is_dup)
    Setting Name of Node

    :Parameters
      :OpenSOAPXMLNodePtr [in, out] ((|node|))
        XML Node Pointer.
      :OpenSOAPStringPtr [in] ((|name|))
        Setting Node Name
      :int [in] ((|is_dup|))
        duplicate flag.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
int
OpenSOAPXMLNodeSetName(OpenSOAPXMLNodePtr /* [in, out] */ node,
                       OpenSOAPStringPtr /* [in] */ name,
                       int /* [in] */ is_dup) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (node) {
        if (is_dup) {
            ret = OpenSOAPStringDuplicate(name, &node->name);
        }
        else {
            if (name) {
                OpenSOAPStringRelease(node->name);
                node->name = name;
                ret = OPENSOAP_NO_ERROR;
            }
        }
    }

    return ret;
}


/*
=begin
--- function#OpenSOAPXMLNodeGetName(node, Name)
    Getting of Node Name

    :Parameters
      :OpenSOAPXMLNodePtr [in] ((|node|))
        XML Node Pointer.
      :OpenSOAPStringPtr [out] ((|name|))
        Result Name of Node.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
int
OpenSOAPXMLNodeGetName(OpenSOAPXMLNodePtr /* [in] */ node,
                       OpenSOAPStringPtr * /* [out] */ name) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (node) {
        ret = OpenSOAPStringDuplicate(node->name, name);
    }

    return ret;
}


/*
=begin
--- function#OpenSOAPXMLNodeInsertNext(node, next)
    Add Next Node to Current Node 

    :Parameters
      :OpenSOAPXMLNodePtr [in, out] ((|node|))
        XML Node.
      :OpenSOAPXMLNodePtr [in, out] ((|next|))
        XML Next Node.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
int
OpenSOAPXMLNodeInsertNext(OpenSOAPXMLNodePtr /* [in, out] */ node,
                          OpenSOAPXMLNodePtr /* [in, out] */ next) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (node) {
        OpenSOAPXMLNodePtr next_tmp = node;
        while (next_tmp->next) {
            next_tmp = next_tmp->next;
        }

        next_tmp->next = next;
        next->prev = next_tmp;

        ret = OPENSOAP_NO_ERROR;
    }

    return ret;
}


/*
=begin
--- function#OpenSOAPXMLNodeRemove(node)
    Remove Node

    :Parameters
      :OpenSOAPXMLNodePtr [in, out] ((|node|))
        XML Node
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
int
OpenSOAPXMLNodeRemove(OpenSOAPXMLNodePtr /* [in, out] */ node){
    int ret = OPENSOAP_YET_IMPLEMENTATION;
    return ret;
}
