/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Spool.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <algorithm>
#include <unistd.h>
#include <iostream>

/* Common Library */
#include "SOAPMessage.h"
#include "FileIDFunc.h"

/* Server Common Library */
#include "ServerCommon.h"
#include "ChannelDescriptor.h"

#include "Spool.h"

using namespace OpenSOAP;

// RXgN^
// FXv[}bṽobNAbvt@C̃pXO
Spool::Spool(char filePass[100]):noEntryMsg("No Entry")
{
  // initiate the string which is returned in the case of no entry

  
  // ƗpMultimapiKey:msgIDCValue:R[h\[o:fileID]j
  MapRecord m;

  string msgID;

  spoolMap.clear();
  strcpy(tableFile, filePass);

  // obNAbvt@C̓ǂݍ
  min.open(tableFile, ios::in);
  if (min.fail()) {
    cout << "Create map file" << endl;
  }
  while (!min.eof() && !min.fail()) {
    min >> msgID;
    min >> m.fileID;
    if (!min.eof()) {
      cout << "init: " << msgID << ", " << m.fileID << endl;
      spoolMap.insert(pair<string, MapRecord>(msgID, m));    
    }
  }
  min.close();

  // Mutex̏
  pthread_mutex_init(&spool_mutex, NULL);
  //pthread_cond_init(&spool_cond, NULL);
  //pthread_create(&pop_thread, NULL, pop, this);
}

// obNAbvt@Co֐
// ԂlF=0Cُ=1
//
int Spool::fileOut()
{
  mout.open(tableFile, ios::out);
  if (mout.fail()) {
    cout << "Map_file output open error" << endl;
    return 1;
  }


  multimap<string, MapRecord>::iterator itr = spoolMap.begin();
  multimap<string, MapRecord>::iterator itrEnd = spoolMap.end();
  for (; itr != itrEnd; itr++) {
    mout << itr->first << "\t" << itr->second.fileID << endl ;
    cout << itr->first << "\t" << itr->second.fileID << endl ;
    mout.flush();
  }
  
  mout.close();
  return 0;
}

// Push\bh
// FbZ[WIDCiID
// ԂlF=0Cُ=0
//
int Spool::push(string msgID, string fileID)
{
  MapRecord m;

  m.fileID = fileID;

  // Mutex̃bN
  int sl = pthread_mutex_lock(&(spool_mutex));
  if (sl != 0) {
    cout << "push thread mutex lock error" << endl;
    return 1;
  }
  else if (sl == 0) {
    cout << "push thread mutex locked" << endl;
  }

  // MultimapւmsgIDƃR[h\̂̑}
  spoolMap.insert(pair<string, MapRecord>(msgID, m)); 

  cout << "map size = " << spoolMap.size() << endl ;

  // obNAbvt@Co
  int sf = fileOut();
  if (sf != 0) {
    cout << "Map file write error" << endl;
    return 2;
  }


  // Mutex̉
  int su = pthread_mutex_unlock(&(spool_mutex));
  if (su != 0) {
    cout << "push thread mutex unlock error" << endl;
    return 3;
  }
  else if (su == 0) {
    cout << "push thread mutex unlocked" << endl;
  }

  return 0;
}

// Pop\bh
// FbZ[WID
// ԂlFiID
//
string Spool::pop(string msgID)
{
  // Ԃl̏ݒ
  string ID = noEntryMsg;
  
  // Mutex̃bN
  int sl = pthread_mutex_lock(&(spool_mutex));
  if (sl == 0) {
    cout << "pop thread mutex locked" << endl;
  }

  // nꂽbZ[WIDKeyƂāCR[hЂƂoD
  // ꂽR[h͏D
  // ȂΕԂstring͏l̂܂܁D
  // 
  multimap<string, MapRecord>::iterator p = spoolMap.find(msgID);
  if (p != spoolMap.end()) {
    ID = p->second.fileID;
    cout << "Poped FileID = " << ID << endl;
    spoolMap.erase(p);
  }
  
  cout << "map size = " << spoolMap.size() << endl ;

  // obNAbvt@Co
  int sf = fileOut();
  if (sf != 0) {
    cout << "Map file write error" << endl;
  }

  // Mutex̉
  int su = pthread_mutex_unlock(&(spool_mutex));
  if (su == 0) {
    cout << "pop thread mutex unlocked" << endl;
  }
  
  // iIDԂ
  return ID;
}

// fXgN^
//
Spool::~Spool()
{
  //pthread_join(pop_thread, NULL);
}

void 
Spool::setChnlDesc(const ChannelDescriptor& chnlDesc)
{
  cerr << "Spool::setChnlDesc" << endl;
  chnlDesc_ = chnlDesc;
}

bool 
Spool::execPushOpr()
{
  cerr << "Spool::execPushOpr()" << endl;
  
  // SrvDrv 瑗Ăf[^ (t@C ID) ǂݍށB
  string fileIDOfMsgFromSrvDrv;
  if (0 > chnlDesc_.read(fileIDOfMsgFromSrvDrv)) {
    cerr << TAG_ERR << "Spool::execPushOpr: read failed" << endl;
    return false;
  }
  
  cerr << "** Spool ** READ From SrvDrv=[" 
       << fileIDOfMsgFromSrvDrv << "]" << endl;
  std::string soapMsgFromSrvDrv = revertFromFileID(fileIDOfMsgFromSrvDrv);
  
  std::string msgIDContent = getMsgIDContent(soapMsgFromSrvDrv);
  // Xv[ɓB
  int ret = push(msgIDContent, fileIDOfMsgFromSrvDrv);
  if (ret != 0) {
    cerr << "Spool::execPushOpr(): Spool push error" << endl;
  }
  
  // Xv[ɓƂ̊mFbZ[W SrvDrv ɕԂB
  string fileIDOfMsgToSrvDrv = "Spool Push success";
  if (0 > chnlDesc_.write(fileIDOfMsgToSrvDrv)) {
    cerr << TAG_ERR << "Spool::execPushOpr: write failed" << endl;
    return false;
  }
  
  cerr << "** Spool ** WRITE To SrvDrv=[" 
       << fileIDOfMsgToSrvDrv << "]" << endl;
  
  return true;
}

bool 
Spool::execPopOpr()
{
  cerr << "Spool::execPopOpr()" << endl;
  
  // MsgDrv 瑗Ăf[^ (MsgID ̃t@C ID) ǂݍށB
  string fileIDOfMsgFromMsgDrv;
  if (0 > chnlDesc_.read(fileIDOfMsgFromMsgDrv)) {
    cerr << TAG_ERR << "Spool::execPopOpr: read failed" << endl;
    return false;
  }
  cerr << "** Spool ** READ From MsgDrv=[" 
       << fileIDOfMsgFromMsgDrv << "]" << endl;
  std::string msgID = revertFromFileID(fileIDOfMsgFromMsgDrv);
  
  cerr << "** Spool ** Contents of ["   
       << fileIDOfMsgFromMsgDrv << "] = " 
       << msgID << endl;
  
  std::string msgIDContent = getMsgIDContent(msgID);
  cerr << "Spool::execPopOpr: getMsgIDContent = " << msgIDContent << endl;
  
  // Xv[oB
  std::string fileIDOfResMsg = pop(msgIDContent);
  
  // In the case of no entry in spool
  // create SOAP fault
  if (fileIDOfResMsg == noEntryMsg) {
    std::string resMsg = makeNoEntryMessage();
    fileIDOfResMsg = convertToFileID(resMsg);
  }
  
  // MsgDrv ɊYMsg ID X|XԂB
  
  if (0 > chnlDesc_.write(fileIDOfResMsg)) {
    cerr << TAG_ERR << "Spool::execPopOpr: write failed" << endl;
    return false;
  }
  
  cerr << "** Spool ** WRITE To MsgDrv=[" 
       << fileIDOfResMsg << "]" << endl;
  
  return true;
}

