/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Timeout.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <cstdio>
#include <iostream>
#include <stdexcept>

#include <pthread.h>
#include <unistd.h>


#include "Timeout.h"

using namespace OpenSOAP;

//===========================================
// RXgN^
//-------------------------------------------
// : ^CAEg(b) 
//===========================================
Timeout::Timeout(int timeout_second)
  : timeoutSecond_(timeout_second)
    , targetThread_(0)
{
  initMutex();
}

//===========================================
// RXgN^
//-------------------------------------------
// : ^CAEg(b) 
// : ^C}[ĎΏۃXbhID
//===========================================
Timeout::Timeout(int timeout_second, pthread_t thr_id)
  : timeoutSecond_(timeout_second)
    , targetThread_(thr_id)
{
  initMutex();
}

//===========================================
// fXgN^
//===========================================
Timeout::~Timeout()
{
  destroyMutex();
}

//===========================================
// mutex
//===========================================
void Timeout::initMutex()
{
  int status = pthread_mutex_init(&isRunning_mutex_, NULL);
  if (0 != status) {
    //throw OpenSOAPException();
    throw std::exception();
  }
}

//===========================================
// mutexj
//===========================================
void Timeout::destroyMutex()
{
  int status = pthread_mutex_init(&isRunning_mutex_, NULL);
  if (0 != status) {
    //throw OpenSOAPException();
    throw std::exception();
  }
}

//===========================================
// ^CAEgΏۃXbhw
//-------------------------------------------
// : ^C}[ĎΏۃXbhID
//===========================================
void Timeout::setTargetThread(pthread_t thr_t)
{
  targetThread_ = thr_t;
}

//===========================================
// ^CAEgԎw(b)
//-------------------------------------------
// : ^CAEg(b) 
//===========================================
void Timeout::setTimeout(int timeout_second)
{
  timeoutSecond_ = timeout_second;
}


//===========================================
// ^C}[
//-------------------------------------------
// : TimeoutCX^X̃|C^
//===========================================
void* Timeout::runTimeout(void* arg)
{
#ifdef DEBUG
  //debug
  cerr << "Timeout::runTimeout() called" << endl;
#endif

  //CX^X擾
  // X^eBbN\bĥ,ɂCX^X
  //   ăCX^XoɃANZX
  Timeout* that = (Timeout*)arg;
  if (NULL == that) {
    //error
    cerr << "invalid argument..." << endl;
  }

  //w莞ԋx~
  sleep(that->timeoutSecond_);

  //-------------------------------
  //ȍ~̓^CAEg
  //-------------------------------
  //  ^CAEgOɏ̏ꍇ͏Lsleep()
  //  finish()Ŗ{Xbh~

  // bNݒ
  int status = pthread_mutex_lock(&(that->isRunning_mutex_));
  if (0 != status) {
    //error
    return NULL;
  }

  //sԃ`FbN
  if (that->isRunning_) {
    //debug
    cerr << "### pthread cancel call!! ###" << endl;

    //s~(^CAEg)
    status = pthread_cancel(that->targetThread_);
    if (0 != status) {
      cerr << "pthread cancel error..." << endl;
    }
    else {
      that->targetThreadIsCanceled_ = true;
    }
  }

#ifdef DEBUG
  // for debug
  else {
    cerr << "## Timeout::runTimeout : not running. may be finished ##" << endl;
  }
#endif

  //bN
  status = pthread_mutex_unlock(&(that->isRunning_mutex_));
  if (0 != status) {
    //error
    return NULL;
  }

#ifdef DEBUG
  //debug
  cerr << "Timeout::runTimeout() return" << endl;
#endif

  return NULL;
}

//===========================================
//^CAEgĎJn
//-------------------------------------------
// ԂlF Xe[^X
//          O:     
//          OȊO: G[
//===========================================
int Timeout::start()
{
#ifdef DEBUG
  //debug
  cerr << "Timeout::start() called" << endl;
#endif

  //ΏۃXbhݒ`FbN
  if (0 == targetThread_) {
    cerr << "target thread not set" << endl;
    return -1;
  }

  // bNݒ
  int status = pthread_mutex_lock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }

  // tOݒ
  isRunning_ = true;
  targetThreadIsCanceled_ = false;
  
  //^C}[XbhJn
  status = pthread_create(&timerThr_id_, 
			   NULL,
			   runTimeout, // static method
			   this);   // static method ɃCX^Xn

  //bN
  status = pthread_mutex_unlock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }

  /*
  //pthread_t thr_id;
  status = pthread_create(&timerThr_id_, NULL,
			   runTimeout, this);
  */

#ifdef DEBUG
  //debug
  cerr << "Timeout::start() return" << endl;
#endif

  return 0;
}

//===========================================
//^CAEgĎI
//-------------------------------------------
// ԂlF Xe[^X
//          O:     
//          OȊO: G[
//===========================================
int Timeout::finish()
{
#ifdef DEBUG
  //debug
  cerr << "Timeout::finish() called" << endl;
#endif

  // bNݒ
  int status = pthread_mutex_lock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }

  // isRunning_ ̃`FbNsȂĂɂ邩???
  if (isRunning_) {
    // ^CAEgOɏIȂ̂,
    // ^C}[Xbh𒆎~
    status = pthread_cancel(timerThr_id_);
    if (0 != status) {
      cerr << "Timeout::runTimeout() thread cancel error" << endl;
    }
    
    // tO
    isRunning_ = false;
  }

  //bN
  status = pthread_mutex_unlock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }

#ifdef DEBUG
  //debug
  cerr << "Timeout::finish() return" << endl;
#endif
  
  return 0;
}
 
// ^[QbgXbh^CAEgŃLZꂽۂH
// 
// true:  LZĂB
// false: LZĂȂB
// 
bool
Timeout::targetThreadIsCanceled()
{
  return targetThreadIsCanceled_;
}
//--- End of Timeout.cpp ---
